// *********************************************************************
//   Universal Gateway Task Function                                   *
// *********************************************************************
//
// File:    UniGWtask.h
// Version: Revision: 1.02
// Date:    2016/08/16
//
// Revision history:
//   2016/07/28  Release for DPA v2.27 IQRF OS v3.08D
//   2016/08/16  RTHOPS == 0 test added
//
// *********************************************************************

// Online Universal gateway documentation: http://www.iqhome.org/products/uni-gw/
// Online DPA documentation http://www.iqrf.org/DpaTechGuide/

#ifndef _UniGWtask_H /* _UniGWtask_H */
#define _UniGWtask_H /* _UniGWtask_H */

//############################################################################################
// Define RF Network Ready Status Pin ( Default Pin: PORTC.2 )
//############################################################################################
#if defined TR52D || defined TR72D
	#define RFnetworkReadyStatusPin( x )	do{ _C2_OUT = x; _C2_TRIS = 0; } while(0)
	#define getRFnetworkReadyStatusPin()	(_C2_OUT)
	#define initUnusedPins()							do{ _C1_OUT = 1; _C1_TRIS = 0; } while(0)
#elif defined TR56D || defined TR76D
	#define RFnetworkReadyStatusPin( x )	do{ _Q15_OUT = x; _Q15_TRIS = 0; } while(0)
	#define getRFnetworkReadyStatusPin()	(_Q15_OUT)
	#define initUnusedPins()							do{ 						\
																				_Q4_OUT = 0;		\
																				_Q5_OUT = 0;		\
																				_Q12_OUT = 0;		\
																				_Q14_OUT = 1;		\
																												\
																				_Q4_TRIS = 0;		\
																				_Q5_TRIS = 0;		\
																				_Q12_TRIS = 0;	\
																				_Q14_TRIS = 0;	\
																				 } while(0)
#else
    #error UniGWtask.h does not correspond to selected TR module type.
#endif

//############################################################################################
// UniGW Task Function
//############################################################################################
// Function description:
// ---------------------
// Visualization of RF network traffic across LEDR and LEDG pins.
//  Green: Sending data to RF network or local DPA command received.
//  Red: Receiving data from RF network or transmitting response of local DPA command to gateway.
//
//  Note: Visualization of RF network traffic can be disabling with defining macro:
//  DISABLE_LEDindication
//  -> Example: #define DISABLE_LEDindication
//
// Set unused pins to output to reduce the power consumption of the module
//  Note: Initialization of unused pins can be disabling with defining macro:
//  DISABLE_initUnusedPins
//  -> Example: #define DISABLE_initUnusedPins
// 																				 
// Controls the RF network is busy status pin (Default pin is: PORTC.2)
//  The pin is set to low after RF package was received from RF network.
//  -> If the pin is low, than the RF network is still routing.
//  The pin is set to high when the timing has expired.
//  -> If the pin is high, than the RF network is ready to send data.
//
// Side effects:
// -------------
// The task controls the LEDR and LEDG pins with pulseLEDR() and pulseLEDG() OS function.
// The task controls the PORTC.2 pin.
// The delay is implemented with DpaTicks global variable. The value of DpaTicks variable is
// modified by this function.
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// !!! Do not use DpaTicks variable in your code !!!
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// FSR0 and FSR1 registers are used as temporary variables.
// Value of FSR0 and FSR1 registers will be destroyed.
// 
// Maximum program memory usage: 90 instruction
//############################################################################################
void UniGWtask();
//############################################################################################

#else /* _UniGWtask_H */

//############################################################################################
#pragma library 1   // Compile only used functions
//############################################################################################

//############################################################################################
void UniGWtask()
//############################################################################################
{
  switch ( GetDpaEvent() )
  {
	  // -------------------------------------------------
	case DpaEvent_Idle:
	  // Do a quick background work when RF packet is not received
	  if ( DpaTicks.15 != 0 )
	  {
		  goto RFnetworkReadyStatusPin_TRUE;	// Set RF Network Ready Status Pin to High
	  }
	  break;

	  // -------------------------------------------------
	case DpaEvent_Init:
	  // Do a one time initialization work before main loop starts
#ifndef DISABLE_LEDindication
	  _systemLEDindication = TRUE;		// Enable LED during special system actions (discovery, FRC)
#endif /* DISABLE_LEDindication */

#ifndef DISABLE_initUnusedPins
	  initUnusedPins();								// Set unused pins to output to reduce power consumption.
#endif

RFnetworkReadyStatusPin_TRUE:
	  RFnetworkReadyStatusPin(TRUE);	// Set RF Network Ready Status Pin to High
	  break;

	  // -------------------------------------------------
	case DpaEvent_ReceiveDpaResponse:
	  // Called after DPA response was received at coordinator
	  FSR0.low8 = RTTSLOT;				// Save RTTSLOT
	  FSR0.high8 = RTHOPS;				// Save RTHOPS
		if( (Zero_) || !(_ROUTEF) ) // RTHOPS == 0 --> Zero_ == 1 
		{
			// This package is not routed package or the routing has been finished
			goto DpaEvent_ReceiveDpaResponse_end;
		}
	  FSR1 = 0;				   // Calculate rest of routing time:
	  do
	  {
	  	FSR1 += FSR0.high8;
	  }
	  while(--FSR0.low8);
	  GIE = FALSE;				   // Disable interrupts)
	  if ( !getRFnetworkReadyStatusPin() && !(DpaTicks.15))
	  {		// Delay still active
	  		if ( DpaTicks > FSR1 )
	  		{	// The existing delay is longer than the new one.
	  			goto DpaEvent_ReceiveDpaResponse_continue;
	  		}
	  }
	  DpaTicks = FSR1;
DpaEvent_ReceiveDpaResponse_continue:
	  GIE = TRUE;				   // Enable interrupts	
	  RFnetworkReadyStatusPin(FALSE);	// Set RF Network Ready Status Pin to Low
DpaEvent_ReceiveDpaResponse_end:
#ifndef DISABLE_LEDindication
	  // -------------------------------------------------
	case DpaEvent_BeforeSendingDpaResponse:
	  // Called before sending DPA response back to originator of DPA response
	case DpaEvent_PeerToPeer:
	  // Called when peer-to-peer (non-networking) packet is received
	  pulseLEDR();				    // Pulse Red LED pulsing
	  break;

	  // -------------------------------------------------
	case DpaEvent_IFaceReceive:
	  // Called after DPA request from interface master was received at coordinator
	  //goto DpaHandleReturnTRUE; // return TRUE to skip default processing
	  pulseLEDG();				// Pulse Green LED pulsing
#endif /* DISABLE_LEDindication */
	  break;
  }
}

//############################################################################################
#pragma library 0   // Compile all
//############################################################################################

//############################################################################################
#endif /* _UniGWtask_H */
//############################################################################################
